# Copyright 2004 SunCERT, Sun Microsystems Inc
# Released under the terms of the GNU General Public License, version 2.
# Written by Ruediger Riediger <ruediger.riediger@sun.com>
#
# MANIFEST: HTML-mail Overlay, allow text/html for transaction content
#
# $Id: RT::Attachment_Local.pm,v 1.0 2005/03/21 zardoz Exp $

no warnings qw(redefine);

use HTML::TreeBuilder;
use HTML::FormatText;

# {{{ sub Content

sub Content {
  my $self = shift;
  my $decode_utf8 = (($self->ContentType eq 'text/plain') ? 1 : 0);
  ### \/ SunCERT: HTML-mail Overlay, store content in $decode \/ ###
  my $decoded;

  if ( $self->ContentEncoding eq 'none' || ! $self->ContentEncoding ) {
      ### \/ SunCERT: HTML-mail Overlay, store content in $decode \/ ###
      $decoded = $self->_Value(
	  'Content',
	  decode_utf8 => $decode_utf8,
      );
  } elsif ( $self->ContentEncoding eq 'base64' ) {
      ### \/ SunCERT: HTML-mail Overlay, store content in $decode \/ ###
      $decoded = ( $decode_utf8
        ? Encode::decode_utf8(MIME::Base64::decode_base64($self->_Value('Content')))
        : MIME::Base64::decode_base64($self->_Value('Content'))
      );
  } elsif ( $self->ContentEncoding eq 'quoted-printable' ) {
      ### \/ SunCERT: HTML-mail Overlay, store content in $decode \/ ###
      $decoded = ( $decode_utf8
        ? Encode::decode_utf8(MIME::QuotedPrint::decode($self->_Value('Content')))
        : MIME::QuotedPrint::decode($self->_Value('Content'))
      );
  } else {
      return( $self->loc("Unknown ContentEncoding [_1]", $self->ContentEncoding));
  }
  ### \/ SunCERT: HTML-mail Overlay, now parse $decode \/ ###
  if($self->ContentType eq 'text/html') {
      my $tree = HTML::TreeBuilder->new->parse( $decoded );
      $tree->eof;

      my $formatter = HTML::FormatText->new(leftmargin => 0);
      # If we somehow fail to do the decode, at least push out the raw bits
      my $html = eval { Encode::decode_utf8( $formatter->format($tree) ) } || $formatter->format($tree);

      $tree = $tree->delete;
      return $html;
  } else {
      return ( $decoded );
  }
  ### /\ SunCERT: HTML-mail Overlay, now parse $decode /\ ###
}

# }}}


# {{{ sub OriginalContent

sub OriginalContent {
  my $self = shift;

  ### \/ SunCERT: HTML-mail Overlay, allow 'html' below \/ ###
  return $self->Content unless $self->ContentType =~ '^text/(plain|html)$';
  my $enc = $self->OriginalEncoding;

  my $content;
  if ( $self->ContentEncoding eq 'none' || ! $self->ContentEncoding ) {
      $content = $self->_Value('Content', decode_utf8 => 0);
  } elsif ( $self->ContentEncoding eq 'base64' ) {
      $content = MIME::Base64::decode_base64($self->_Value('Content', decode_utf8 => 0));
  } elsif ( $self->ContentEncoding eq 'quoted-printable' ) {
      return MIME::QuotedPrint::decode($self->_Value('Content', decode_utf8 => 0));
  } else {
      return( $self->loc("Unknown ContentEncoding [_1]", $self->ContentEncoding));
  }

  # Encode::_utf8_on($content);
  if (!$enc || $enc eq '' ||  $enc eq 'utf8' || $enc eq 'utf-8') {
    # If we somehow fail to do the decode, at least push out the raw bits
    eval {return( Encode::decode_utf8($content))} || return ($content);
  }
  
  eval { Encode::from_to($content, 'utf8' => $enc);};
  if ($@) {
	$RT::Logger->error("Could not convert attachment from assumed utf8 to '$enc' :".$@);
  }
  return $content;
}

# }}}


# {{{ sub Quote 

sub Quote {
    my $self=shift;
    my %args=(Reply=>undef, # Prefilled reply (i.e. from the KB/FAQ system)
	      @_);

    my ($quoted_content, $body, $headers);
    my $max=0;

    # TODO: Handle Multipart/Mixed (eventually fix the link in the
    # ShowHistory web template?)
    ### \/ SunCERT: HTML-mail Overlay, allow 'html' below \/ ###
    if ($self->ContentType =~ m{^(text/(plain|html)$|message/)}i) {
	$body=$self->Content;

	# Do we need any preformatting (wrapping, that is) of the message?

	# Remove quoted signature.
	$body =~ s/\n-- \n(.*)$//s;

	# What's the longest line like?
	foreach (split (/\n/,$body)) {
	    $max=length if ( length > $max);
	}

	if ($max>76) {
	    require Text::Wrapper;
	    my $wrapper=new Text::Wrapper
		(
		 columns => 70, 
		 body_start => ($max > 70*3 ? '   ' : ''),
		 par_start => ''
		 );
	    $body=$wrapper->wrap($body);
	}

	$body =~ s/^/> /gm;

	$body = '[' . $self->TransactionObj->CreatorObj->Name() . ' - ' . $self->TransactionObj->CreatedAsString()
	            . "]:\n\n"
   	        . $body . "\n\n";

    } else {
	$body = "[Non-text message not quoted]\n\n";
    }
    
    $max=60 if $max<60;
    $max=70 if $max>78;
    $max+=2;

    return (\$body, $max);
}

# }}}

1;

